@@ -2,7 +2,8 @@
  * AT-SPI - Assistive Technology Service Provider Interface
  * (Gnome Accessibility Project; http://developer.gnome.org/projects/gap)
  *
- * Copyright 2001 Sun Microsystems Inc.
+ * Copyright 2001, 2002 Sun Microsystems Inc.,
+ * Copyright 2001, 2002 Ximian, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -57,7 +58,7 @@ impl_isValid (PortableServer_Servant _servant,
 BONOBO_TYPE_FUNC_FULL (SpiHyperlink,
 		       Accessibility_Hyperlink,
 		       SPI_TYPE_BASE,
-		       spi_hyperlink);
+		       spi_hyperlink)
 
 
 static void
@@ -83,7 +84,7 @@ spi_hyperlink_init (SpiHyperlink *hyperlink)
 
 
 SpiHyperlink *
-spi_hyperlink_new (AtkObject *object)
+spi_hyperlink_new (AtkHyperlink *object)
 {
   SpiHyperlink *new_hyperlink = g_object_new (
 	  SPI_HYPERLINK_TYPE, NULL);
@@ -100,7 +101,7 @@ get_hyperlink_from_servant (PortableServer_Servant servant)
   SpiBase *object = SPI_BASE (bonobo_object_from_servant (servant));
 
   g_return_val_if_fail (object != NULL, NULL);
-  g_return_val_if_fail (ATK_IS_OBJECT(object->gobj), NULL);
+  g_return_val_if_fail (ATK_IS_HYPERLINK(object->gobj), NULL);
   return ATK_HYPERLINK (object->gobj);
 }
 
@@ -113,7 +114,7 @@ impl__get_n_anchors (PortableServer_Servant servant,
 
   g_return_val_if_fail (link != NULL, 0);
 
-  return (CORBA_short) atk_hyperlink_get_n_anchors (link);
+  return atk_hyperlink_get_n_anchors (link);
 }
 
 
@@ -125,7 +126,7 @@ impl__get_startIndex (PortableServer_Servant servant,
 
   g_return_val_if_fail (link != NULL, -1);
 
-  return (CORBA_long) atk_hyperlink_get_start_index (link);
+  return atk_hyperlink_get_start_index (link);
 }
 
 
@@ -137,7 +138,7 @@ impl__get_endIndex (PortableServer_Servant servant,
 
   g_return_val_if_fail (link != NULL, -1);
 
-  return (CORBA_long) atk_hyperlink_get_end_index (link);
+  return atk_hyperlink_get_end_index (link);
 }
 
 
@@ -151,7 +152,7 @@ impl_getURI (PortableServer_Servant servant,
 
   g_return_val_if_fail (link != NULL, CORBA_string_dup (""));
 
-  uri = atk_hyperlink_get_uri (link, (gint) i);
+  uri = atk_hyperlink_get_uri (link, i);
   if (uri)
     {
       rv = CORBA_string_dup (uri);
@@ -174,7 +175,7 @@ impl_getObject (PortableServer_Servant servant,
 
   g_return_val_if_fail (link != NULL, CORBA_OBJECT_NIL);
 
-  atk_object = atk_hyperlink_get_object (link, (gint) i);
+  atk_object = atk_hyperlink_get_object (link, i);
 
   return spi_accessible_new_return (atk_object, FALSE, ev);
 }
@@ -188,6 +189,6 @@ impl_isValid (PortableServer_Servant servant,
 
   g_return_val_if_fail (link != NULL, TRUE);
 
-  return (CORBA_boolean) atk_hyperlink_is_valid (link);
+  return atk_hyperlink_is_valid (link);
 }
 
